@@ -19,18 +19,12 @@ from cubes.vcsfile import queries
 
 def rql_revision_content(repoeid, revnum, branch=None):
     """return rql query to get the repository content at a given revision"""
-    # XXX: MAX(V) is a trick necessary because RQL doesn't support subqueries,
-    #      which would be necessary to have the correct query. This trick is ok
-    #      since we know that entity with MAX(V) has MAX(VR) as well, due to
-    #      source implementation
     args = {'rev': revnum, 'x': repoeid, 'branch': branch}
-    return ('Any MAX(V),MAX(REV),F,D,N GROUPBY D,N,F ORDERBY D,N '
-            'WHERE V content_for F, V from_revision VR, VR revision REV, '
-            'VR branch %(branch)s, '
-            'F directory D, F name N, F from_repository R, R eid %(x)s, '
-            'F is VersionedFile, (VR revision <= %(rev)s AND NOT EXISTS('
-            'X is DeletedVersionContent, X content_for F, X from_revision XR, '
-            'XR branch %(branch)s, XR revision < %(rev)s, XR revision >= REV))'
+    return ('Any V,REV,F,D,N ORDERBY D,N WHERE '
+            'V content_for F, F directory D, F name N, '
+            'V from_revision VR, VR revision REV, '
+            'V at_revision AVR, AVR revision %(rev)s, AVR branch %(branch)s, '
+            'AVR from_repository R, R eid %(x)s'
             ), args
 
 _MARKER = object()
@@ -106,9 +100,11 @@ class Repository(AnyEntity):
 
     def branches(self):
         """return existing branches"""
-        return [b for b, in self._cw.execute(
-            'DISTINCT Any B WHERE R eid %(r)s, REV from_repository R, '
-            'REV branch B', {'r' : self.eid})]
+        rql = 'DISTINCT Any B WHERE R eid %(r)s, REV from_repository R, REV branch B'
+        branches = [b for b, in self._cw.execute(rql, {'r' : self.eid})]
+        if not branches:
+            branches = [self.default_branch()]
+        return branches
 
     @cached
     def branch_head(self, branch=_MARKER):
@@ -116,10 +112,12 @@ class Repository(AnyEntity):
         if branch is _MARKER:
             branch = self.default_branch()
         rset = self._cw.execute(
-            'Any MAX(REV) WHERE V at_revision REV, REV branch %(branch)s, '
-            'REV from_repository R, R eid %(r)s',
+            'Any REV WHERE REV from_repository R, R eid %(r)s, REV revision RR '
+            'WITH RR BEING (Any MAX(RR) WHERE '
+            '  REV from_repository R, R eid %(r)s, REV branch %(branch)s, '
+            '  REV revision RR)',
             {'r': self.eid, 'branch': branch})
-        if rset[0][0] is None:
+        if not rset:
             return None
         return rset.get_entity(0, 0)
 
@@ -130,15 +128,6 @@ class Repository(AnyEntity):
             {'repo' : self.eid, 'dir' : directory, 'name' : filename})
         return rset and rset.get_entity(0, 0) or None
 
-    def is_directory_deleted(self, directory):
-        # XXX same MAX(X) trick as above
-        rset = self._cw.execute(
-            'Any MAX(VC), MAX(REV) GROUPBY VF '
-            'WHERE VC from_revision R, R revision REV, R from_repository X, '
-            'X eid %(x)s, VC content_for VF, VF directory ~= %(dir)s',
-            {'x': self.eid, 'dir': directory})
-        return not any(e for e in rset.entities() if e.__regid__ == 'VersionContent')
-
     # vcs write support ########################################################
 
     def default_branch(self):
@@ -280,9 +269,13 @@ class VersionedFile(AnyEntity):
     def head(self):
         """return latest [deleted] version content for this file"""
         rset = self._cw.execute(
-            'Any MAX(V) WHERE X eid %(x)s, V content_for X', {'x': self.eid})
+            'Any V WHERE X eid %(x)s, V content_for X, V at_revision R, '
+            'R revision RR WITH RR BEING '
+            '(Any MAX(RR) WHERE X eid %(x)s, V content_for X, V at_revision R, '
+            ' R revision RR)',
+            {'x': self.eid})
         # content_for relation is not mandatory on VersionedFile entities
-        if rset[0][0] is None:
+        if not rset:
             return None
         return rset.get_entity(0, 0)
 
@@ -306,11 +299,13 @@ class VersionedFile(AnyEntity):
         if branch is _MARKER:
             branch = self.repository.default_branch()
         rset = self._cw.execute(
-            'Any MAX(V) WHERE X eid %(x)s, V content_for X, V at_revision REV, '
-            'REV branch %(branch)s',
+            'Any V WHERE X eid %(x)s, V content_for X, V at_revision R, '
+            'R revision RR WITH RR BEING '
+            '(Any MAX(RR) WHERE X eid %(x)s, V content_for X, V at_revision R, '
+            ' R branch %(branch)s, R revision RR)',
             {'x': self.eid, 'branch': branch})
         # content_for relation is not mandatory on VersionedFile entities
-        if rset[0][0] is None:
+        if not rset:
             return None
         return rset.get_entity(0, 0)
 
@@ -409,9 +404,9 @@ class DeletedVersionContent(AnyEntity):
     def previous_versions(self):
         return tuple(self._cw.execute(
             'DISTINCT Any VC ORDERBY VC DESC WHERE '
-            'R parent_revision PR, X at_revision R, VC at_revision PR, '
-            'VC content_for F, X content_for F, '
-            'NOT VC eid %(x)s, X eid %(x)s', {'x': self.eid}).entities())
+            'X from_revision R, X content_for F, X eid %(x)s, '
+            'VC at_revision PR, VC content_for F, NOT VC eid %(x)s, '
+            'R parent_revision PR', {'x': self.eid}).entities())
 
     @deprecated('[vcsfile 1.7.1] use previous_versions() instead')
     def previous_version(self, branch=_MARKER):
@@ -421,9 +416,9 @@ class DeletedVersionContent(AnyEntity):
     def next_versions(self):
         return tuple(self._cw.execute(
             'DISTINCT Any VC ORDERBY VC ASC WHERE '
-            'PR parent_revision R, X at_revision R, VC at_revision PR, '
-            'VC content_for F, X content_for F, '
-            'NOT VC eid %(x)s, X eid %(x)s', {'x': self.eid}).entities())
+            'X at_revision R, X content_for F, X eid %(x)s, '
+            'VC from_revision PR, VC content_for F, NOT VC eid %(x)s, '
+            'PR parent_revision R', {'x': self.eid}).entities())
 
     @deprecated('[vcsfile 1.7.1] use next_versions() instead')
     def next_version(self, branch=_MARKER):
