@@ -36,10 +36,7 @@ should be commited (default to 1, e.g. on each revision).',
         ('import-revision-content',
          {'type' : 'yn',
           'default': True,
-          'help': 'Import content of file touched by a revision. This may \
-cause performance issue when scaling. Should be set correctly even if the \
-instance is not the instance responsible to import repositories\' content \
-(e.g. repository-import=no).',
+          'help': 'This option is now deprecated in favor of the import_revision_content attribute per Repository.',
           'level': 2,
           'group': 'vcsfile',
           }),
