@@ -159,7 +159,10 @@ class VFUploadFormView(form.FormViewMixIn, EntityView):
     __select__ = is_instance('VersionedFile')
 
     submitmsg = _('new revision has been checked-in')
-    action = 'vcsnewrev'
+
+    @property
+    def action(self):
+        return self._cw.build_url('vcsnewrev')
 
     def call(self, **_kwargs):
         # XXX explain why we would want self.rset.rowcount != 1
