@@ -26,6 +26,7 @@ _abaa = uicfg.actionbox_appearsin_addmenu
 _pvs.tag_subject_of(('*', 'at_revision', '*'), 'hidden')
 _pvs.tag_object_of(('*', 'at_revision', '*'), 'hidden')
 _pvs.tag_attribute(('Repository', 'local_cache'), 'hidden')
+_pvs.tag_attribute(('Repository', 'source_url'), 'hidden') # custom view
 _pvs.tag_object_of(('*', 'from_repository', 'Repository'), 'hidden')
 
 _pvs.tag_attribute(('VersionedFile', 'name'), 'hidden') # in title
@@ -74,11 +75,10 @@ class RepositoryInfoTab(tabs.PrimaryTab):
     __regid__ = 'repositoryinfo_tab'
     __select__ = is_instance('Repository')
 
-    def render_entity_title(self, entity):
-        pass
-
     def render_entity_attributes(self, entity):
         super(RepositoryInfoTab, self).render_entity_attributes(entity)
+        if entity.source_url:
+            entity.view('vcsfile.repository.checkout', w=self.w)
         rset = self._cw.execute('Any REV, REVB, REVN ORDERBY REVN DESC '
                                 'WHERE REV branch REVB, REV revision REVN, REV from_repository R, '
                                 'R eid %(r)s, NOT X parent_revision REV',
@@ -119,6 +119,27 @@ class RepositoryHistoryTab(EntityView):
                    paginate=True, displayfilter=True, displaycols=range(5))
 
 
+class RepositoryCheckOutInstructionsView(EntityView):
+    __regid__ = 'vcsfile.repository.checkout'
+    __select__ = is_instance('Repository') & score_entity(lambda x: x.source_url)
+
+    def entity_call(self, entity):
+        w = self.w
+        _ = self._cw._
+        w(u'<h3>%s</h3>' % _('Browse source'))
+        w(u'<p>%s</p>' % _('You can browse the source code by following <a href="%s">this link</a>.')
+          % xml_escape(entity.source_url))
+        w(u'<h3>%s</h3>' % _('Command-Line Access'))
+        w(u'<p>%s</p>' % _('Non-members may check out a read-only working copy anonymously over HTTP.'))
+        w(u'<p>%s</p>' % _('Use this command to check out the latest project source code:'))
+        w(u'<pre>')
+        if entity.type == 'mercurial':
+            w(u'hg clone %s' % xml_escape(entity.source_url))
+        else: # svn
+            w(u'svn checkout %s' % xml_escape(entity.source_url))
+        w(u'</pre>')
+
+
 class DVCPrimaryView(primary.PrimaryView):
     __select__ = is_instance('DeletedVersionContent')
 
